package com.biganto.visual.roompark.presentation.screen.feeds

import android.view.View
import android.widget.TextView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.domain.model.FeedModel
import com.google.android.material.tabs.TabLayout
import com.jakewharton.rxbinding3.material.selections
import io.reactivex.Observable
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class FeedsScreenController  :
    BigantoBaseController<FeedsScreenViewState
        , FeedsScreen
        , FeedsScreenPresenter>()
    , FeedsScreen {

    override fun feedsTabSelected(): Observable<Int>
            = feedsTabs.selections().map { it.position }
        .debounce (200L, TimeUnit.MILLISECONDS)
        .map { storedFeedsList[it].feedId }


    override fun injectDependencies() {
        getComponent()
    }

    @BindView(R.id.feedsTabs)
    lateinit var feedsTabs:TabLayout


    private var storedFeedsList:List<FeedModel> = arrayListOf()

    @Inject
    override lateinit var injectedPresenter: FeedsScreenPresenter


    override fun onViewBound(v: View) {

    }

    override fun render(viewState: FeedsScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is FeedsScreenViewState.Idle -> render(viewState)
            is FeedsScreenViewState.FeedsPages -> render(viewState)
            is FeedsScreenViewState.GetFeedArticlesPreview -> render(viewState)
        }
    }

    private fun render(viewState: FeedsScreenViewState.Idle){

    }

    private fun render(viewState: FeedsScreenViewState.FeedsPages){
        feedsTabs.removeAllTabs()

        viewState.items.forEach {feed ->
            val tab = feedsTabs.newTab()
                .setCustomView(R.layout.feeds_tab_view)
            tab.customView
                ?.let {
                    it.findViewById<TextView>(R.id.tab_title)?.text = feed.title

                    it.findViewById<TextView>(R.id.tab_divider)?.visibility =
                        if (viewState.items.indexOf(feed) == viewState.items.size - 1) View.GONE
                        else View.VISIBLE
                }
            feedsTabs.addTab(tab)
        }
        storedFeedsList = viewState.items
    }


    private fun render(viewState: FeedsScreenViewState.GetFeedArticlesPreview){
        snackbar.showSnackBar(viewState.items.first().announce)
    }

    private fun getComponent() = DaggerFeedsScreenComponent.factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

    override fun getLayoutId(): Int = R.layout.feeds_screen


}